/*
 * alestis_event_classifier
 *
 * Beacons event classifier for Alestis.
 *
 * Copyright(c) 2019 BeOneOff Digital Solutions S.L.
 * All rights reserved.
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited.
 * Proprietary and confidential.
 *
 * Authors:
 * Alberto Castillo <alberto.castillo@beoneoff.com>
 */

'use-strict';

// Third party modules
const schedule = require('node-schedule');

// Project modules
const {
  currentBeaconOrNewBeacon,
  stateTransaction,
  isBeaconKeepAlive,
} = require('./src/services/beacon.service');

// Parameters
const {
  enterToAliveDelay,
  leaveToEnterDelay,
  aliveToLeaveDelay,
  thresholdMax,
  thresholdMin,
  cronKeepAliveInterval,
} = require('./parameters');

const beaconArray = [];

// Arrays for Classifier the state of beacons
const enterBeacons = [];
const aliveBeacons = [];
const leaveBeacons = [];

// RETURN BEACON ARRAY
function getBeaconArray() {
  const beacons = beaconArray.map(beacon => beacon.getBeacon());
  return beacons;
}

function executeClassifier(msgPayload) {
  const data = JSON.parse(msgPayload);
  const currentBeacon = currentBeaconOrNewBeacon(beaconArray, data);
  stateTransaction(
    currentBeacon,
    leaveToEnterDelay,
    enterToAliveDelay,
    enterBeacons,
    aliveBeacons,
    leaveBeacons,
  );

  schedule.scheduleJob(cronKeepAliveInterval, () => {
    isBeaconKeepAlive(aliveBeacons, leaveBeacons, thresholdMax, thresholdMin, aliveToLeaveDelay);
  });

  return getBeaconArray();
}

module.exports = {
  executeClassifier,
  getBeaconArray,
};
