/*
 * alestis_event_classifier
 *
 * Beacons event classifier for Alestis.
 *
 * Copyright(c) 2019 BeOneOff Digital Solutions S.L.
 * All rights reserved.
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited.
 * Proprietary and confidential.
 *
 * Authors:
 * Alberto Castillo <alberto.castillo@beoneoff.com>
 */

('use-strict');

const KalmanFilter = require('kalmanjs');

class BeaconModel {
  constructor(id, lastSeen) {
    this.id = id;
    this.rawRSSI = [0, 0, 0, 0, 0];
    this.kalmanRSSI = [0, 0, 0, 0, 0];
    this.lastSeen = lastSeen;
    this.state = 'ENTER';
    this.kalman = new KalmanFilter({ R: 0.1, Q: 10 });
  }

  getBeacon() {
    const result = {
      ID: this.id,
      KALMAN_RSSI: this.kalmanRSSI,
      LAST_SEEN: this.lastSeen,
      STATE: this.state,
    };
    return result;
  }

  getLastSeen() {
    return this.lastSeen;
  }

  updateLastSeen(lastSeen) {
    this.lastSeen = lastSeen;
  }

  addRSSI(rawRSSI) {
    // FIRST ELEMENT OUT
    this.rawRSSI.shift();
    this.rawRSSI.push(rawRSSI);
    const rssiFiltered = this.kalman.filter(rawRSSI);
    // FIRST ELEMENT OUT
    this.kalmanRSSI.shift();
    this.kalmanRSSI.push(rssiFiltered);
  }

  setState(state) {
    this.state = state;
  }
}

module.exports = BeaconModel;
